@@ -1,7 +1,7 @@
 #include "config.h"
 
 #include <glib.h>
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
 #include <locale.h>
 #include <glib-object.h>
 #include "gdm-languages.h"
@@ -12,6 +12,8 @@ int main (int argc, char **argv)
 	guint i;
 
 	setlocale (LC_ALL, NULL);
+	textdomain (GETTEXT_PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
 	g_type_init ();
