@@ -74,6 +74,7 @@ add_details_row (GtkWidget *details, gint top, const gchar *heading, const gchar
         heading_label = gtk_label_new (heading);
         gtk_style_context_add_class (gtk_widget_get_style_context (heading_label), "dim-label");
         gtk_widget_set_halign (heading_label, GTK_ALIGN_END);
+        gtk_widget_set_valign (heading_label, GTK_ALIGN_START);
         gtk_widget_set_hexpand (heading_label, TRUE);
 
         gtk_grid_attach (GTK_GRID (details), heading_label, 0, top, 1, 1);
@@ -132,21 +133,25 @@ add_details (GtkWidget *details, NMDevice *device, NMConnection *connection)
 {
         NMIPConfig *ip4_config = NULL;
         NMIPConfig *ip6_config = NULL;
-        gchar *ip4_address = NULL;
-        gchar *ip4_route = NULL;
-        gchar *ip4_dns = NULL;
-        gchar *ip6_address = NULL;
+        g_autofree gchar *ip4_address = NULL;
+        g_autofree gchar *ip4_route = NULL;
+        g_autofree gchar *ip4_dns = NULL;
+        g_autofree gchar *ip6_address = NULL;
+        g_autofree gchar *ip6_route = NULL;
+        g_autofree gchar *ip6_dns = NULL;
         gint i = 0;
 
         ip4_config = nm_device_get_ip4_config (device);
         if (ip4_config) {
                 ip4_address = panel_get_ip4_address_as_string (ip4_config, "address");
                 ip4_route = panel_get_ip4_address_as_string (ip4_config, "gateway");
-                ip4_dns = panel_get_ip4_dns_as_string (ip4_config);
+                ip4_dns = panel_get_dns_as_string (ip4_config);
         }
         ip6_config = nm_device_get_ip6_config (device);
         if (ip6_config) {
-                ip6_address = panel_get_ip6_address_as_string (ip6_config);
+                ip6_address = panel_get_ip6_address_as_string (ip6_config, "address");
+                ip6_route = panel_get_ip6_address_as_string (ip6_config, "gateway");
+                ip6_dns = panel_get_dns_as_string (ip6_config);
         }
 
         if (ip4_address && ip6_address) {
@@ -155,28 +160,35 @@ add_details (GtkWidget *details, NMDevice *device, NMConnection *connection)
         } else if (ip4_address) {
                 add_details_row (details, i++, _("IP Address"), ip4_address);
         } else if (ip6_address) {
-                add_details_row (details, i++, _("IPv6 Address"), ip6_address);
+                add_details_row (details, i++, _("IP Address"), ip6_address);
         }
 
         add_details_row (details, i++, _("Hardware Address"),
                          nm_device_ethernet_get_hw_address (NM_DEVICE_ETHERNET (device)));
 
-        if (ip4_route)
+        if (ip4_route && ip6_route) {
+                g_autofree gchar *ip_routes = g_strjoin ("\n", ip4_route, ip6_route, NULL);
+                add_details_row (details, i++, _("Default Route"), ip_routes);
+        } else if (ip4_route) {
                 add_details_row (details, i++, _("Default Route"), ip4_route);
-        if (ip4_dns)
+        } else if (ip6_route) {
+                add_details_row (details, i++, _("Default Route"), ip6_route);
+        }
+        
+        if (ip4_dns && ip6_dns) {
+                add_details_row (details, i++, _("DNS4"), ip4_dns);
+                add_details_row (details, i++, _("DNS6"), ip6_dns);
+        } else if (ip4_dns) {
                 add_details_row (details, i++, _("DNS"), ip4_dns);
-
-        if (nm_device_get_state (device) != NM_DEVICE_STATE_ACTIVATED) {
-                gchar *last_used;
-                last_used = get_last_used_string (connection);
-                add_details_row (details, i++, _("Last used"), last_used);
-                g_free (last_used);
+        } else if (ip6_dns) {
+                add_details_row (details, i++, _("DNS"), ip6_dns);
         }
 
-        g_free (ip4_address);
-        g_free (ip4_route);
-        g_free (ip4_dns);
-        g_free (ip6_address);
+        if (nm_device_get_state (device) != NM_DEVICE_STATE_ACTIVATED) {
+                g_autofree gchar *last_used = NULL;
+                last_used = get_last_used_string (connection);
+                add_details_row (details, i++, _("Last used"), last_used);
+        }
 }
 
 static void populate_ui (NetDeviceEthernet *device);
@@ -202,7 +214,7 @@ device_ethernet_refresh_ui (NetDeviceEthernet *device)
         NMDevice *nm_device;
         NMDeviceState state;
         GtkWidget *widget;
-        gchar *speed = NULL;
+        g_autofree gchar *speed = NULL;
 
         nm_device = net_device_get_nm_device (NET_DEVICE (device));
 
